package jadex.adapter.jade;

import jade.lang.acl.*;
import jade.content.ContentManager;
import jade.content.onto.basic.Action;
import jadex.util.SReflect;

/**
 *  Match an acl message by specifying the extracted content classes.
 */
public class MatchOntologyContent implements MessageTemplate.MatchExpression
{
	//-------- attributes -------

	/** The content manager. */
	protected ContentManager manager;

	/** The content object class. */
	protected Class	contentclass;

	/** The agent action class (contained in action object). */
	protected Class	actionclass;

	//-------- constrcutors --------

	/**
	 *  Create a new match start content literal.
	 *  (case-insensitiv).
	 */
	public MatchOntologyContent(ContentManager manager, Class contentclass, Class actionclass)
	{
		this.manager = manager;
		this.contentclass = contentclass;
		this.actionclass = actionclass;
	}

	/**
	 *  Test if the content begins with some string.
	 * @param message The message.
	 * @return True, if it begins with the string.
	 */
	public boolean match(ACLMessage message)
	{
		boolean ret = false;
		// Extract message content (if any).
		if(message.getOntology()!=null && message.getLanguage()!=null)
		{
			try
			{
				Object content = manager.extractContent(message);
				//System.out.println("msg: "+cont);

				ret = true;
				if(contentclass!=null)
				{
					ret	= content!=null
						&& SReflect.isSupertype(contentclass, content.getClass());
				}
				if(ret && actionclass!=null)
				{
					ret	= (content instanceof Action)
						&& SReflect.isSupertype(actionclass, ((Action)content).getAction().getClass());
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}

		return ret;
	}

	/**
	 *  Get the string representation.
	 *  @return The string representation.
	 */
	public String toString()
	{
		return  SReflect.getInnerClassName(this.getClass());
	}
}
