package jadex.adapter.jade;

import jade.core.behaviours.CyclicBehaviour;
import jadex.adapter.IJadexAgent;

/**
 *  The jadex Timing Behaviour is responsible for
 *  waiting specified amounts of time and then waking
 *  up the next timeouted behaviour.
 */
public class TimingBehaviour extends CyclicBehaviour
{
	//-------- attributes --------

	/** The agent. */
	protected IJadexAgent agent;

	/** The next-wakeup time. */
	protected long next_wakeup;

	//-------- constructor --------

	/**
	 *  Create a new TimingBehaviour.
	 *  @param agent The jadex agent.
	 */
	public TimingBehaviour(IJadexAgent agent)
	{
		this.agent	= agent;
	}

	//-------- methods --------

	/**
	 *  Wait till the next timeout.
	 */
	public void action()
	{
		if(next_wakeup==0)
		{
			block();
			//System.out.println("blocked (0)");
		}
		else
		{
			long diff	= next_wakeup - System.currentTimeMillis();

			if(diff>0)
			{
				block(diff);
				//System.out.println("blocked: "+diff);
			}
			else //if(diff<=0)
			{
				block();
				next_wakeup = 0;
				agent.notifyDue();
				//System.out.println("blocked (0): 2");
			}
		}
	}

	/**
	 *  Set the next wake-up time.
	 */
	protected void notifyIn(long millis)
	{
		if(millis>-1)
		{
			this.next_wakeup = System.currentTimeMillis() + millis;
			this.restart();
			//System.out.println("timing restarted");
		}
		else
		{
			this.next_wakeup	= 0;
		}
	}
}

