package jadex.adapter.jade.planlib;

import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.Agent;
import jade.core.ContainerID;
import jade.domain.JADEAgentManagement.CreateAgent;
import jade.domain.JADEAgentManagement.JADEManagementOntology;
import jade.wrapper.AgentContainer;
import jade.wrapper.AgentController;
import jade.wrapper.StaleProxyException;
import jadex.adapter.fipa.AgentIdentifier;
import jadex.adapter.jade.JadeAgentAdapter;
import jadex.adapter.jade.SJade;
import jadex.model.SXML;
import jadex.runtime.IGoal;
import jadex.runtime.Plan;

import java.util.Iterator;
import java.util.Map;

/**
 *  Plan for starting a Jadex agent on JADE.
 */
public class JadeAMSCreateAgentPlan extends Plan
{
	//-------- constants --------

	/** The agent counter. */
	public static int agentcnt = 0;

	//-------- methods --------

	/**
	 *  Execute a plan.
	 */
	public void body()
	{
		// todo: support starting in another container?!
		String	clazz	= JadeAgentAdapter.class.getName();
		String	type	= (String)getParameter("type").getValue();
		String	name	= (String)getParameter("name").getValue();
		boolean	start	= ((Boolean)getParameter("start").getValue()).booleanValue();
		if(name==null)
			name = SXML.getShortName(type)+agentcnt++;
		String	config	= (String)getParameter("configuration").getValue();
		if(config==null)
			config	= "";	// Hack!!! Bug in JADE does not allow null in sl sequence.
		AgentIdentifier	amsfipa	= (AgentIdentifier)getParameter("ams").getValue();

		// If ams is specified use remote code.
		if(amsfipa!=null)
		{
			if(!start)
				throw new RuntimeException("JADE adapter can only create remote agents with start==true.");
			
			AID	amsjade	= SJade.convertAIDtoJade(amsfipa);
			CreateAgent	create	= new CreateAgent();
			create.setClassName(clazz);
			create.setAgentName(name);
			create.setContainer(new ContainerID("Main-Container", null)); // Hack!!!
			create.addArguments(type);
			create.addArguments(config);
			Map	args	= (Map)getParameter("arguments").getValue();
			if(args!=null)
			{
				for(Iterator it=args.keySet().iterator(); it.hasNext(); )
				{
					Object	key	= it.next();
					if(!(key instanceof String))
						throw new RuntimeException("Remote agent creation supports only strings as argument names: "+key);
					Object	value	= args.get(key);
					if(!(value instanceof String))
						throw new RuntimeException("Remote agent creation supports only strings as argument value: "+value);
					create.addArguments(key+"="+value);
				}
			}

			IGoal	req	= createGoal("rp_initiate");
			req.getParameter("receiver").setValue(amsfipa);
			req.getParameter("action").setValue(new Action(amsjade, create));
			req.getParameter("ontology").setValue(JADEManagementOntology.NAME);
			req.getParameter("language").setValue(jade.domain.FIPANames.ContentLanguage.FIPA_SL0);
			dispatchSubgoalAndWait(req);
			
			// Hack!!! Bug in JADE not returning created agent's AID.
			// Should do ams_search do get correct AID?
			AID	ret	= (AID)amsjade.clone();
			int idx	= ret.getName().indexOf("@");
			ret.setName(name+ret.getName().substring(idx));
			getParameter("agentidentifier").setValue(SJade.convertAIDtoFipa(ret));
		}
		
		// Otherwise create agent locally.
		else
		{
			try
			{
				Object[]	args	= new Object[]{type, config, getParameter("arguments").getValue()};
				AgentContainer container = ((Agent)getScope().getPlatformAgent()).getContainerController();
				AgentController	controller = container.createNewAgent(name, clazz, args);
				if(start)
					controller.start();
	
				// Hack!!! Does not provide addresses, etc.
				AID	aid	= new AID(controller.getName(), AID.ISGUID);
				getParameter("agentidentifier").setValue(SJade.convertAIDtoFipa(aid));
			}
			catch(StaleProxyException e)
			{
				throw new RuntimeException("Error creating agent "+type+"\n", e);
			}
		}
	}
}
