package jadex.adapter.jade.tools.logger.gui;

import jade.gui.AgentTree;
import jade.gui.StringDlg;
import jade.tools.sniffer.PopupMouser;


import javax.swing.*;
import javax.swing.text.JTextComponent;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.border.Border;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.util.Vector;

import jadex.adapter.jade.tools.logger.Logger;
import jadex.adapter.jade.tools.logger.gui.AgentList;
import jadex.adapter.jade.tools.logger.gui.LogTablePanel;
import jadex.util.jtable.IVisibilityTableColumnModel;
import jadex.util.jtable.VisibilityTableColumnModel;

public class MainPanel extends JPanel
{
    public static final String FILTER_TEXT = "Set filter (you can use regular expressions):";
    public static final String PRELOAD_TEXT = "Preload=";

    public AgentTree treeAgent;
    public LogTablePanel logTable;
    public FilterTablePanel filterTable;
    public AgentList listAgent;

    public PreloadPanel preloadPanel;

    private JTextPane filterText;


    private JSplitPane pane;
    private JSplitPane leftpane;
    private JSplitPane rightpane;
    private JPanel panel;
    private PopupMouser popM;
    private Font font = new Font("Helvetica", Font.PLAIN, 12);
    int pos;

    public MainPanel(Logger myLogger, MainWindow mwnd)
    {
        Font f = new Font("SanSerif", Font.PLAIN, 14);
        setFont(f);
        setLayout(new BorderLayout(10, 10));

        preloadPanel = new PreloadPanel(myLogger);

        treeAgent = new AgentTree(f);
        listAgent = new AgentList(f);
        logTable = new LogTablePanel(mwnd, this, myLogger);


        filterTable = new FilterTablePanel(mwnd, this, myLogger, logTable);
        filterTable.table.setTableHeader(null);

        // Hack !!!
        // add the logTable's jtable to columnmodel as listener after columnmodel
        // is added to filerTable's jtable. otherwise resizing is impossible,
        // because the latest added columnmodellistener (jtable) will get the
        // risizingColumn from its header and filterTable's jtable doesnt have one.
        logTable.table.getColumnModel().removeColumnModelListener(logTable.table);
        filterTable.table.setColumnModel(logTable.table.getColumnModel());
        logTable.table.getColumnModel().addColumnModelListener(logTable.table);

        filterText = new JTextPane();

        filterText.setBackground(Color.lightGray);
        filterText.setFont(font);
        filterText.setText(FILTER_TEXT);
        filterText.setEditable(false);

        final JScrollPane scrollLog = new JScrollPane(logTable.table, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        final JScrollPane scrollFilter = new JScrollPane(filterTable.table, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        JScrollBar barLog = scrollLog.getHorizontalScrollBar();
        JScrollBar barFilter = scrollFilter.getHorizontalScrollBar();
        barFilter.setModel(barLog.getModel());
        scrollFilter.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        barLog.getModel().addChangeListener(new ChangeListener()
        {

            public void stateChanged(ChangeEvent e)
            {
                final Dimension log = scrollLog.getViewport().getExtentSize();
                final Dimension filter = scrollFilter.getViewport().getExtentSize();
                if (log.getWidth() != filter.getWidth())
                {
                    if (log.getWidth() > filter.getWidth())
                    {
                        scrollFilter.getViewport().setExtentSize(new Dimension(log.width, filter.height));
                    }
                }
            }
        });

        panel = new JPanel(new BorderLayout());
        panel.add(filterText, BorderLayout.NORTH);
        panel.add(scrollFilter, BorderLayout.CENTER);


        leftpane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, new JScrollPane(treeAgent.tree), new JScrollPane(listAgent.list));
        leftpane.setContinuousLayout(true);


        rightpane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, scrollLog, panel);
        rightpane.setContinuousLayout(true);

        pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftpane, rightpane);
        pane.setContinuousLayout(true);

        add(pane);
        add(preloadPanel,BorderLayout.SOUTH);

        treeAgent.tree.addMouseListener(new PopupMouser(treeAgent.tree, treeAgent));

        logTable.table.addMouseListener(new MyMouseListener());

        filterTable.tableModel.addTableModelListener(new MyTableModelListener());


    }

    private void refreshFilterText()
    {
        Vector filter = logTable.getFilter();
        Vector concat = new Vector();
        String result = new String();

        for (int i = 0; i < filter.size(); i++)
        {
            Vector rowFilter = (Vector) filter.elementAt(i);
            for (int j = 0; j < rowFilter.size(); j++)
            {
                if (rowFilter.elementAt(j) != null)
                {
                    String cellFilter = rowFilter.elementAt(j).toString().trim();
                    //                String col = logTable.tableModel.columnNames[j];
                    String col = logTable.tableModel.getColumnName(j);
                    if (cellFilter.length() != 0)
                    {
                        String text = col + "=" + cellFilter;
                        concat.addElement(text);
                    }
                }
            }
            if (result.length() != 0 && concat.size() != 0)
            {
                result = result + " OR ";
            }
            for (int j = 0; j < concat.size(); j++)
            {
                boolean lastElement = j < concat.size() - 1;
                boolean firstElement = j == 0;
                String s = (String) concat.elementAt(j);
                result = result + (firstElement ? "( " : "") + s + (lastElement ? " AND " : " )");
            }
            concat.removeAllElements();
        }

        if (result.length() == 0)
        {
            filterText.setText(FILTER_TEXT);
        }
        else
        {
            filterText.setText(result);
        }

    }

    public void adjustDividerLocation()
    {
        pane.setDividerLocation(0.3);
        leftpane.setDividerLocation(0.6);
        rightpane.setDividerLocation(0.8);
    }

    public void setFilterVisible(boolean newFilterVisible)
    {
        rightpane.getBottomComponent().setVisible(newFilterVisible);

        if (newFilterVisible)
        {
            rightpane.setDividerSize(10);
            rightpane.setDividerLocation(0.8);
            //            rightpane.setLastDividerLocation(rightpane.getDividerLocation());
        }
        else
        {
            rightpane.setDividerSize(0);
        }

    }

    public boolean isFilterVisible()
    {
        return rightpane.getBottomComponent().isVisible();
    }

    public void clearFilter()
    {
        filterTable.removeAllRecords();
    }

    private class MyTableModelListener implements TableModelListener
    {
        public void tableChanged(TableModelEvent e)
        {
            logTable.refresh();
            refreshFilterText();
        }
    }

    private class MyMouseListener implements MouseListener
    {
        public void mouseClicked(MouseEvent e)
        {
            // check for double click
            if ((e.getButton() == MouseEvent.BUTTON1) && (e.getClickCount() == 2))
            {
                int row = logTable.table.rowAtPoint(e.getPoint());
                int column = logTable.table.columnAtPoint(e.getPoint());
                String cell = logTable.table.getValueAt(row, column).toString();
                //TODO: schow a JDialog with JTextField
                JOptionPane.showMessageDialog(null, cell, "Content of current cell", JOptionPane.INFORMATION_MESSAGE);
            }

        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
    }

}
