package jadex.adapter.jade.tools.rmaplugin;

import jade.content.lang.sl.SLCodec;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.Agent;
import jade.core.ContainerID;
import jade.domain.JADEAgentManagement.CreateAgent;
import jade.domain.JADEAgentManagement.JADEManagementOntology;
import jade.lang.acl.ACLMessage;
import jade.wrapper.AgentController;
import jade.wrapper.StaleProxyException;
import jadex.adapter.fipa.SFipa;
import jadex.runtime.IMessageEvent;
import jadex.tools.common.plugin.IControlCenter;
import jadex.tools.common.plugin.IMessageListener;
import jadex.tools.jcc.AbstractJCCPlugin;
import jadex.tools.jcc.ControlCenter;
import jadex.util.SGUI;
import jadex.util.SUtil;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.*;

/**
 *  A plugin that displays the JADE RMA window as a panel in the JCC.
 */
public class RMAPlugin	extends AbstractJCCPlugin
{
	//-------- constants --------
	
	/** The image icons. */
	protected static final UIDefaults icons = new UIDefaults(new Object[]
	{
		"rmaplugin", SGUI.makeIcon(RMAPlugin.class, "/jadex/adapter/jade/tools/rmaplugin/images/logosmall.png"),
		"rmaplugin_sel", SGUI.makeIcon(RMAPlugin.class, "/jadex/adapter/jade/tools/rmaplugin/images/logosmall_sel.png"),
	});

	//-------- attributes --------
	
	/** The rma toolbar. */
	protected JToolBar	toolbar;
	
	/** The rma menus. */
	protected JMenuBar	menubar;
	
	/** The rma plugin agent. */
	protected AgentController	rma;

	//-------- constructors --------
	
	/**
	 *  Create the rma plugin.
	 */
	public void	init(final IControlCenter jcc)
	{
		super.init(jcc);

		try
		{
			String	id	= jcc.getAgent().getName()+"_rmaplugin";
			PLUGINS.put(id, this);
			
			// Problem: does not provide addresses JADE bug
			/*this.rma	= ((Agent)jcc.getAgent().getPlatformAgent()).getContainerController()
				.createNewAgent(id, "jadex.adapter.jade.tools.rmaplugin.RMAPluginAgent", new Object[]{id});
			rma.start();*/
			
			AID ams = ((Agent)jcc.getAgent().getPlatformAgent()).getAMS();
			Agent agent = (Agent)jcc.getAgent().getPlatformAgent();
			
			CreateAgent	create	= new CreateAgent();
			Action action = new Action(ams, create);
			create.setClassName("jadex.adapter.jade.tools.rmaplugin.RMAPluginAgent");
			create.setAgentName(id);
			create.setContainer(new ContainerID("Main-Container", null)); // Hack!!!
			create.addArguments(id);

			ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
			msg.addReceiver(ams);
			msg.setOntology(JADEManagementOntology.NAME);
			msg.setLanguage(jade.domain.FIPANames.ContentLanguage.FIPA_SL0);
			agent.getContentManager().registerLanguage(new SLCodec(0));
			agent.getContentManager().registerOntology(JADEManagementOntology.getInstance());
			agent.getContentManager().fillContent(msg, action);
			agent.send(msg);
			
			jcc.addMessageListener(new IMessageListener()
			{
				public boolean processMessage(IMessageEvent msg)
				{
					boolean	handled	= false;
					if(SFipa.INFORM.equals(msg.getParameter(SFipa.PERFORMATIVE).getValue())
						&& "rma_closed".equals(msg.getContent()))
					{
						handled	= true;
						SwingUtilities.invokeLater(new Runnable()
						{
							public void run()
							{
								try
								{
									jcc.getAgent().killAgent();
								}
								catch(Exception e)
								{
								}
							}
						});
					}
					return handled;
				}
			});
		}
		catch(Exception e)
		{
			String	message	= "Could not create rma plugin agent:\n";
			Throwable	ex	= e;
			while(ex!=null)
			{
				if(ex.getMessage()!=null)
					message	+= ex.getMessage()+"\n";
				ex	= ex.getCause();
			}
			String text = SUtil.wrapText(message);
			// Hack!!! Shouldn't access window.
			JOptionPane.showMessageDialog(((ControlCenter)jcc).getWindow(), text, "Error initializing RMA plugin", JOptionPane.ERROR_MESSAGE);
			getView().add(BorderLayout.CENTER, new JTextArea(text));
		}
	}

	//-------- IControlCenterPlugin interface --------

	/**
	 *  The name of the plugin.
	 */
	public String getName()
	{
		return "RMAPlugin";
	}

	/**
	 *  Return the held id referencing the chapter in the online help.
	 */
	public String getHelpID()
	{
		return "tools.rma";
	}

	/**
	 *  Return the icon of the plugin. 
	 */
	public Icon getToolIcon(boolean selected)
	{
		return selected? icons.getIcon("rmaplugin_sel"): icons.getIcon("rmaplugin");
	}
		
	/**
	 *  Get the panel.
	 *  @return The panel.
	 */
	public JComponent createView()
	{
		return new JPanel(new BorderLayout());
	}
	
	/**
	 *  Close the plugin.
	 */
	public void shutdown()
	{
		if(rma!=null)
		{
			try
			{
				rma.kill();
			}
			catch(StaleProxyException e)
			{
				// Ignore???
			}
		}
	}
		
	//-------- inter agent part --------
	
	/**
	 *  This method is used to exchange the panel object between the two agents (rma/jcc).
	 */
	public void	setGUI(final MainWindow mw)
	{
		// Get components from gui.
		while(mw.getContentPane().getComponentCount()>0)
		{
			Component	comp	= mw.getContentPane().getComponent(0);
			mw.getContentPane().remove(0);
			if(comp instanceof JToolBar)
			{
				toolbar	= (JToolBar)comp;
				getView().add(BorderLayout.NORTH, toolbar);
				getView().validate();
				getView().repaint();
			}
			else if(comp instanceof MainPanel)
			{
				getView().add(BorderLayout.CENTER, comp);
				getView().validate();
				getView().repaint();
			}
		}
		
		// Get menus.
		this.menubar	= mw.getJMenuBar();
		for(int i=0; i<menubar.getMenuCount(); i++)
			menubar.getMenu(i).setText(menubar.getMenu(i).getText()+"-Jade");

		// Set the location of the main window as it is used to calculate positions of dialogs.
		// Hack!!! Shouldn't access window.
		JFrame	window	= ((ControlCenter)getJCC()).getWindow();
		mw.setBounds(window.getBounds());
		window.addComponentListener(new ComponentAdapter()
		{
			public void componentMoved(ComponentEvent e)
			{
				mw.setBounds(((Component)e.getSource()).getBounds());
			}
			public void componentResized(ComponentEvent e)
			{
				mw.setBounds(((Component)e.getSource()).getBounds());
			}
		});
	}

	/** The map containing the plugins by identifier. */
	public static Map	PLUGINS	= new Hashtable();
}
